<#
 /**
 * Copyright(c) 2006-2016 ThreatSTOP, Inc. All Rights Reserved
 * NOTICE: All information contained herein is, and remains the property of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and
 * technical concepts contained herein are proprietary to ThreatSTOP, Inc. and its suppliers and certain aspects thereof are protected by United
 * States Patent No. 8,533,822 and United States Patent No. 8,869,237, and are protected by trade secret or copyright law.
 * US Government Entities: The ThreatSTOP service, software and documentation, as applicable, are "commercial computer software" and
 * "commercial computer software documentation" developed exclusively at private expense by Threatstop, Inc. ("Threatstop"). Pursuant to FAR
 * 12.212 or DFARS 227 7202 and their successors, as applicable, use, reproduction and disclosure of the software and documentation is
 * governed by the terms of Threatstop's commercial agreement.
 */
#>
# ErrID start: 20
# Takes a specified zone (from ini file) and convert it into a number of query policy blocks
# 1000 domains per policy.

#Load config values into hash table:
$scriptPath = split-path -parent $MyInvocation.MyCommand.Definition
$iniPath = (get-item $scriptPath).parent.fullname
Get-Content "$iniPath\ThreatSTOP.ini" | foreach-object -begin {$h=@{}} -process { $k = [regex]::split($_,'='); if(($k[0].CompareTo("") -ne 0) -and ($k[0].StartsWith("[") -ne $True -and $k[0].StartsWith("#")  -ne $True)) { $h.Add($k[0].Trim(), $k[1].Trim()) } }

$zonefolder = $h.TSZoneFolder
$tszonename = $h.TSZoneName
$dns_action = $h.DNS_Action

#test if zone file exists:
$domtxt=($zonefolder + "\" + $tszonename + ".dns")
if (Test-Path $domtxt)
{
    Write-Host("Using $domtxt file")
}
else
{
    Write-Host ("$domtxt not found. Exiting.")
    break;
}
if ((Get-Item $domtxt).length -lt 5kb )
{
    #include first line of zone file when reporting error
    $fl = ""
    try{
        $fl = Get-Content $domtxt -First 1
    }
    catch{}
    $msg = "$domtxt file error. Zone may not have transferred from ThreatSTOP. Check your ThreatSTOP portal, device IP address, and network time. Additional info: $fl"
    Write-EventLog -LogName Application -Source 'ThreatSTOP' -EntryType Error -EventId 21 -Message $msg
    Write-Host $msg
    break;
}

$curmaxTSpol=-1
Get-DnsServerQueryResolutionPolicy | Foreach-object -process {if (($_.name -match "ThreatSTOPblock(\d+)") -and ($curmaxTSpol -lt ($($matches[1]) -as [int]))) { $curmaxTSpol= ($($matches[1]) -as [int])}}
$passthru = New-Object System.Collections.ArrayList
$passthru.Add('EQ') > $null
$i=$pol=0
$reader = [System.IO.File]::OpenText($domtxt)

try {
    $out = New-Object System.Collections.ArrayList
    $out.Add('EQ') > $null
    for(;;) {
        $line = $reader.ReadLine()
        if ($line -eq $null) { break }
        # process the line

        # Only check actual RPZ records
        if (-Not ($line -match "\sCNAME\s")) {
            continue
        }
        if ($line -match "^((?:\*\.)?[a-z0-9][\w\.-]+)\s+\d+\s+\w+") {
            #Write-Host ("Line $line")
            $dom=$($matches[1])
            if ($dom -match "rpz-[a-z]+$") {
                #Write-Host ("Skipping dom $dom");
                continue
            }
            if ($dom -match "^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\.(rpz-ip)|(rpz-nsip)\.*") {
                #Write-Host ("Skipping IP $dom");
                continue
            }
            if ($line -match "CNAME\s*rpz-passthru\.*")
            {
                #passthrus go in a separate collection.
                #Write-Host ("Skipping passthru $dom")
                $passthru.Add(($dom -replace "$tszonename.", "")) > $null
                continue
            }
            #Write-Host ("Adding $dom")
            $i++
            $out.Add(($dom -replace "$tszonename.", "")) > $null #remove policyname from dig entries. no replacements made when using dns secondary zone file

            if ($i -ge 1000) {
                if ($pol -le $curmaxTSpol) {
                    Set-DnsServerQueryResolutionPolicy -Name "ThreatSTOPblock$pol" -Fqdn ($out -join "," )
                } else {
                    Add-DnsServerQueryResolutionPolicy -Name "ThreatSTOPblock$pol" -Action $dns_action -Fqdn ($out -join "," )
                }
                #Write-Host("ThreatSTOPblock$pol")
                $i=0
                $pol++
                $out = New-Object System.Collections.ArrayList
                $out.Add("EQ") > $null
            }
        } else {
        #Write-Host ("Skipping line $line")
        }
    }
    if ($pol -le $curmaxTSpol) {
        Set-DnsServerQueryResolutionPolicy -Name "ThreatSTOPblock$pol" -Fqdn ($out -join "," )
    } else {
        Add-DnsServerQueryResolutionPolicy -Name "ThreatSTOPblock$pol" -Action $dns_action -Fqdn ($out -join "," )
    }

    #passthrus are skipped

    $msg = ("Policies updated. Last TS pol: ThreatSTOPblock$pol")
    Write-EventLog -LogName Application -Source 'ThreatSTOP' -EntryType Information -EventId 20 -Message $msg
    $pol++
}
finally {
    $reader.Close()
    for (; $pol -le $curmaxTSpol; $pol++) {
        Write-Host("Removing old pol: ThreatSTOPblock$pol")
        Remove-DnsServerQueryResolutionPolicy -Name "ThreatSTOPblock$pol" -Force
    }
    #Get-DnsServerQueryResolutionPolicy | Foreach-object -process {Write-host($_.name)}
}
